@@ -59,7 +59,7 @@ extern HANDLE afsi_file;
  */
 static void afsd_notifier(char *msgp, char *filep, long line)
 {
-	char tbuffer[100];
+	char tbuffer[512];
 	char *ptbuf[1];
 	HANDLE h;
 
@@ -104,7 +104,7 @@ static void afsd_notifier(char *msgp, char *filep, long line)
 /*
  * For use miscellaneously in smb.c; need to do better
  */
-static int DummyMessageBox(HWND h, LPCTSTR l1, LPCTSTR l2, UINT ui)
+static int _stdcall DummyMessageBox(HWND h, LPCTSTR l1, LPCTSTR l2, UINT ui)
 {
 	return 0;
 }
@@ -146,11 +146,65 @@ afsd_ServiceFlushVolume(DWORD dwlpEventData)
     return dwRet;
 }
 
+
+/* service control handler used in nt4 only for backward compat. */
+VOID WINAPI 
+afsd_ServiceControlHandler(DWORD ctrlCode)
+{
+	HKEY parmKey;
+	DWORD dummyLen, doTrace;
+	long code;
+
+	switch (ctrlCode) {
+		case SERVICE_CONTROL_STOP:
+			/* Shutdown RPC */
+			RpcMgmtStopServerListening(NULL);
+
+			/* Force trace if requested */
+			code = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
+					    AFSConfigKeyName,
+					    0, KEY_QUERY_VALUE, &parmKey);
+			if (code != ERROR_SUCCESS)
+				goto doneTrace;
+
+			dummyLen = sizeof(doTrace);
+			code = RegQueryValueEx(parmKey, "TraceOnShutdown",
+						NULL, NULL,
+						(BYTE *) &doTrace, &dummyLen);
+			RegCloseKey (parmKey);
+			if (code != ERROR_SUCCESS)
+				doTrace = 0;
+			if (doTrace)
+				afsd_ForceTrace(FALSE);
+
+doneTrace:
+			ServiceStatus.dwCurrentState = SERVICE_STOP_PENDING;
+			ServiceStatus.dwWin32ExitCode = NO_ERROR;
+			ServiceStatus.dwCheckPoint = 1;
+			ServiceStatus.dwWaitHint = 10000;
+			ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+			SetServiceStatus(StatusHandle, &ServiceStatus);
+			SetEvent(WaitToTerminate);
+			break;
+		case SERVICE_CONTROL_INTERROGATE:
+			ServiceStatus.dwCurrentState = SERVICE_RUNNING;
+			ServiceStatus.dwWin32ExitCode = NO_ERROR;
+			ServiceStatus.dwCheckPoint = 0;
+			ServiceStatus.dwWaitHint = 0;
+			ServiceStatus.dwControlsAccepted = SERVICE_ACCEPT_STOP;
+			SetServiceStatus(StatusHandle, &ServiceStatus);
+			break;
+		/* XXX handle system shutdown */
+		/* XXX handle pause & continue */
+	}
+}
+
+
 /*
 **    Extended ServiceControlHandler that provides Event types
 **    for monitoring Power events, for example.
 */
-DWORD
+DWORD WINAPI
 afsd_ServiceControlHandlerEx(
               DWORD  ctrlCode,
               DWORD  dwEventType,
@@ -253,6 +307,7 @@ afsd_ServiceControlHandlerEx(
  * Mount a drive into AFS if there global mapping
  */
 /* DEE Could check first if we are run as SYSTEM */
+#define MAX_RETRIES 30
 static void MountGlobalDrives()
 {
     char szAfsPath[_MAX_PATH];
@@ -260,7 +315,7 @@ static void MountGlobalDrives()
     DWORD dwResult;
     char szKeyName[256];
     HKEY hKey;
-    DWORD dwIndex = 0;
+    DWORD dwIndex = 0, dwRetry = 0;
     DWORD dwDriveSize;
     DWORD dwSubMountSize;
     char szSubMount[256];
@@ -272,7 +327,7 @@ static void MountGlobalDrives()
 	if (dwResult != ERROR_SUCCESS)
         return;
 
-    while (1) {
+    while (dwRetry < MAX_RETRIES) {
         dwDriveSize = sizeof(szDriveToMapTo);
         dwSubMountSize = sizeof(szSubMount);
         dwResult = RegEnumValue(hKey, dwIndex++, szDriveToMapTo, &dwDriveSize, 0, &dwType, szSubMount, &dwSubMountSize);
@@ -284,6 +339,7 @@ static void MountGlobalDrives()
             }
         }
 
+        for ( ; dwRetry < MAX_RETRIES; dwRetry++)
 		{
 		    NETRESOURCE nr;
 		    memset (&nr, 0x00, sizeof(NETRESOURCE));
@@ -298,8 +354,14 @@ static void MountGlobalDrives()
 		    nr.dwUsage = RESOURCEUSAGE_CONNECTABLE;
 
 		    dwResult = WNetAddConnection2(&nr,NULL,NULL,FALSE);
-		}
-        afsi_log("GlobalAutoMap of %s to %s %s", szDriveToMapTo, szSubMount, dwResult ? "succeeded" : "failed");
+            afsi_log("GlobalAutoMap of %s to %s %s (%d)", szDriveToMapTo, szSubMount, 
+                     (dwResult == NO_ERROR) ? "succeeded" : "failed", dwResult);
+            if (dwResult == NO_ERROR) {
+                break;
+            }
+            /* wait for smb server to come up */
+            Sleep((DWORD)1000 /* miliseconds */);		
+        }
     }        
 
     RegCloseKey(hKey);
@@ -350,6 +412,16 @@ typedef BOOL ( APIENTRY * AfsdInitHook )(void);
 #define AFSD_INIT_HOOK "AfsdInitHook"
 #define AFSD_HOOK_DLL  "afsdhook.dll"
 
+/*
+control serviceex exists only on 2000/xp. These functions will be loaded dynamically.
+*/
+
+typedef SERVICE_STATUS_HANDLE ( * RegisterServiceCtrlHandlerExFunc )(  LPCTSTR , LPHANDLER_FUNCTION_EX , LPVOID );
+typedef SERVICE_STATUS_HANDLE ( * RegisterServiceCtrlHandlerFunc   )(  LPCTSTR ,  LPHANDLER_FUNCTION );
+
+RegisterServiceCtrlHandlerExFunc pRegisterServiceCtrlHandlerEx = NULL;
+RegisterServiceCtrlHandlerFunc   pRegisterServiceCtrlHandler   = NULL; 
+
 void afsd_Main(DWORD argc, LPTSTR *argv)
 {
 	long code;
@@ -358,6 +430,7 @@ void afsd_Main(DWORD argc, LPTSTR *argv)
 	int jmpret;
 #endif /* JUMP */
     HANDLE hInitHookDll;
+    HANDLE hAdvApi32;
     AfsdInitHook initHook;
 
 #ifdef _DEBUG
@@ -377,10 +450,23 @@ void afsd_Main(DWORD argc, LPTSTR *argv)
         afsi_log("Event Object Already Exists: %s", TEXT("afsd_service_WaitToTerminate"));
 
 #ifndef NOTSERVICE
-	StatusHandle = RegisterServiceCtrlHandlerEx(argv[0] /* AFS_DAEMON_SERVICE_NAME */,
-			(LPHANDLER_FUNCTION_EX) afsd_ServiceControlHandlerEx,
-                                                 NULL /* user context */
-                                                 );
+    hAdvApi32 = LoadLibrary("advapi32.dll");
+    if (hAdvApi32 == NULL)
+    {
+        afsi_log("Fatal: cannot load advapi32.dll");
+        return;
+    }
+
+    pRegisterServiceCtrlHandlerEx = (RegisterServiceCtrlHandlerExFunc)GetProcAddress(hAdvApi32, "RegisterServiceCtrlHandlerExA");
+    if (pRegisterServiceCtrlHandlerEx)
+    {
+        afsi_log("running on 2000+ - using RegisterServiceCtrlHandlerEx");
+        StatusHandle = RegisterServiceCtrlHandlerEx(AFS_DAEMON_SERVICE_NAME, afsd_ServiceControlHandlerEx, NULL );
+    }
+    else
+    {
+        StatusHandle = RegisterServiceCtrlHandler(AFS_DAEMON_SERVICE_NAME, afsd_ServiceControlHandler);
+    }
 
 	ServiceStatus.dwServiceType = SERVICE_WIN32_OWN_PROCESS;
 	ServiceStatus.dwServiceSpecificExitCode = 0;
@@ -417,7 +503,8 @@ void afsd_Main(DWORD argc, LPTSTR *argv)
             hookRc = initHook();
         }
         FreeLibrary(hInitHookDll);
-               
+        hInitHookDll = NULL;
+
         if (hookRc == FALSE)
         {
             ServiceStatus.dwCurrentState = SERVICE_STOPPED;
@@ -474,7 +561,7 @@ void afsd_Main(DWORD argc, LPTSTR *argv)
         ServiceStatus.dwWaitHint -= 5000;
         SetServiceStatus(StatusHandle, &ServiceStatus);
 #endif
-		code = afsd_InitSMB(&reason, DummyMessageBox);
+		code = afsd_InitSMB(&reason, MessageBox);
 		if (code != 0) {
             afsi_log("afsd_InitSMB failed: %s (code = %d)", reason, code);
 			osi_panic(reason, __FILE__, __LINE__);
@@ -524,23 +611,18 @@ void afsd_Main(DWORD argc, LPTSTR *argv)
     /* Remove the ExceptionFilter */
     SetUnhandledExceptionFilter(NULL);
 
-    if ( hInitHookDll )
-        FreeLibrary(hInitHookDll);
-
-    Sleep(5000);
-
     ServiceStatus.dwCurrentState = SERVICE_STOPPED;
 	ServiceStatus.dwWin32ExitCode = GlobalStatus ? ERROR_EXCEPTION_IN_SERVICE : NO_ERROR;
 	ServiceStatus.dwCheckPoint = 0;
 	ServiceStatus.dwWaitHint = 0;
 	ServiceStatus.dwControlsAccepted = 0;
 	SetServiceStatus(StatusHandle, &ServiceStatus);
-
 }
 
 DWORD __stdcall afsdMain_thread(void* notUsed)
 {
-	afsd_Main(0, (LPTSTR*)NULL);
+    char * argv[2] = {AFS_DAEMON_SERVICE_NAME, NULL};
+    afsd_Main(1, (LPTSTR*)argv);
     return(0);
 }
 
