@@ -98,7 +98,7 @@ void cm_CheckServers(long flags, cm_cell_t *cellp)
 	                                code = RXAFS_GetTime(connp->callp, &secs, &usecs);
 	                        }
 				if (wasDown)
-					rx_SetConnDeadTime(connp->callp, CM_CONN_CONNDEADTIME);
+					rx_SetConnDeadTime(connp->callp, ConnDeadtimeout);
 	                        cm_PutConn(connp);
 			}	/* got an unauthenticated connection to this server */
 
@@ -199,17 +199,21 @@ cm_server_t *cm_NewServer(struct sockaddr_in *socketp, int type, cm_cell_t *cell
 	osi_assert(socketp->sin_family == AF_INET);
 
 	tsp = malloc(sizeof(*tsp));
-        memset(tsp, 0, sizeof(*tsp));
+    memset(tsp, 0, sizeof(*tsp));
 	tsp->type = type;
-        tsp->cellp = cellp;
-        tsp->refCount = 1;
-	tsp->allNextp = cm_allServersp;
-        cm_allServersp = tsp;
+    tsp->cellp = cellp;
+    tsp->refCount = 1;
 	lock_InitializeMutex(&tsp->mx, "cm_server_t mutex");
 	tsp->addr = *socketp;
 
 	cm_SetServerPrefs(tsp); 
-        return tsp;
+
+    lock_ObtainWrite(&cm_serverLock); /* get server lock */
+	tsp->allNextp = cm_allServersp;
+    cm_allServersp = tsp;
+    lock_ReleaseWrite(&cm_serverLock); /* release server lock */
+
+    return tsp;
 }
 
 /* find a server based on its properties */
@@ -326,15 +330,14 @@ long cm_ChangeRankServer(cm_serverRef_t** list, cm_server_t*	server)
 		}
 		current = & ( (*current)->next);	
 	}
-	/* if this volume is not replicated on this server  */
-	if (!element) {
-        lock_ReleaseWrite(&cm_serverLock);
+    lock_ReleaseWrite(&cm_serverLock);
+
+    /* if this volume is not replicated on this server  */
+	if (!element)
 		return 1;	/* server is not on list */
-    }
 
 	/* re-insert deleted element into the list with modified rank*/
 	cm_InsertServerList(list, element);
-    lock_ReleaseWrite(&cm_serverLock);
 	return 0;
 }
 /*
@@ -387,6 +390,33 @@ void cm_RandomizeServer(cm_serverRef_t** list)
     lock_ReleaseWrite(&cm_serverLock);
 }
 
+/* call cm_FreeServer while holding a write lock on cm_serverLock */
+void cm_FreeServer(cm_server_t* server)
+{
+    if (--(server->refCount) == 0)
+    {
+        /* we need to check to ensure that all of the connections
+         * for this server have a 0 refCount; otherwise, they will
+         * not be garbage collected 
+         */
+        cm_GCConnections(&server);  /* connsp */
+
+        lock_FinalizeMutex(&server->mx);
+        if ( cm_allServersp == server )
+            cm_allServersp = server->allNextp;
+        else {
+            cm_server_t *tsp;
+
+            for(tsp = cm_allServersp; tsp->allNextp; tsp=tsp->allNextp) {
+                if ( tsp->allNextp == server ) {
+                    tsp->allNextp = server->allNextp;
+                    break;
+                }
+            }
+        }
+    }
+ }
+
 void cm_FreeServerList(cm_serverRef_t** list)
 {
     cm_serverRef_t  *current = *list;
@@ -396,9 +426,10 @@ void cm_FreeServerList(cm_serverRef_t** list)
 
     while (current)
     {
-	   next = current->next;
-	   free(current);
-	   current = next;
+        next = current->next;
+        cm_FreeServer(current->server);
+        free(current);
+        current = next;
     }
   
     lock_ReleaseWrite(&cm_serverLock);
